package org.zeroturnaround.javarebel.integration.servlet;

import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.Plugin;
import org.zeroturnaround.javarebel.ServletIntegration;
import org.zeroturnaround.javarebel.integration.generic.BaseServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.generic.ServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.servlet.jasper.CompilerCBP;
import org.zeroturnaround.javarebel.integration.servlet.jasper.JasperLoaderCBP;
import org.zeroturnaround.javarebel.integration.servlet.jasper.JspRuntimeContextCBP;
import org.zeroturnaround.javarebel.integration.servlet.jasper.JspServletCBP;
import org.zeroturnaround.javarebel.integration.servlet.jetty.JettyContextHandlerCBP;
import org.zeroturnaround.javarebel.integration.servlet.jetty.JettyServletHandlerCBP;
import org.zeroturnaround.javarebel.integration.servlet.oc4j.HttpApplicationCBP;
import org.zeroturnaround.javarebel.integration.servlet.oc4j.OC4JJspClassLoaderCBP;
import org.zeroturnaround.javarebel.integration.servlet.oc4j.OC4JJspJavacCompilerCBP;
import org.zeroturnaround.javarebel.integration.servlet.oc4j.ServletDescriptorCBP;
import org.zeroturnaround.javarebel.integration.servlet.oc4j.ServletMappingCBP;
import org.zeroturnaround.javarebel.integration.servlet.resin.PageCBP;
import org.zeroturnaround.javarebel.integration.servlet.resin.ResinApplicationCBP;
import org.zeroturnaround.javarebel.integration.servlet.sap.SapApplicationContext;
import org.zeroturnaround.javarebel.integration.servlet.sap.SapServletContextImplCBP;
import org.zeroturnaround.javarebel.integration.servlet.tomcat.ApplicationContextFacadeCBP;
import org.zeroturnaround.javarebel.integration.servlet.tomcat.WebappLoaderCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.JavelinxTagFileStaleCheckerCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.JspConfigCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.JspStubCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.ServletStubImplCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.TagFileHelperCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.WebAppConfigManagerCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.WebAppServletContextCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.WeblogicCPLCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.WeblogicJspClassLoaderCBP;
import org.zeroturnaround.javarebel.integration.servlet.weblogic.WeblogicTagFileClassLoaderCBP;
import org.zeroturnaround.javarebel.integration.servlet.websphere.AbstractJSPExtensionServletWrapperCBP;
import org.zeroturnaround.javarebel.integration.servlet.websphere.JspOptionsCBP;
import org.zeroturnaround.javarebel.integration.servlet.websphere.WebappCBP;

/**
 * Servlet integration JavaRebel plugin.
 * 
 * @author Rein Raudjärv
 * 
 * @see ServletIntegration
 */
public class ServletIntegrationPlugin implements Plugin {
  public void preinit() {
    ClassLoader cl = getClass().getClassLoader();

    // SERVLET CONTEXT
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "javax.servlet.ServletContext", 
        new ServletContextCBP());
    
    // JETTY 5.x (Tested on 5.1.14)
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.servlet.ServletHandler",
        new JettyServletHandlerCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.servlet.ServletHandler$Context",
        new BaseServletContextImplClassBytecodeProcessor());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.servlet.Default",
        new DefaultServletCBP());

    // JETTY 6.x (Tested on 6.1.12)
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.handler.ContextHandler",
        new JettyContextHandlerCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.handler.ContextHandler$SContext",
        new BaseServletContextImplClassBytecodeProcessor());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.mortbay.jetty.servlet.DefaultServlet",
        new DefaultServletCBP());
    
    /*
     * TOMCAT 4.1, 5.5, 6.0 (Tested on 4.1.37, 5.5.27, 6.0.14)
     * GLASSFISH 2.1 (Tested on 2.1-b60e)
     * JBOSS 4.2, 5.0 (Tested on 4.2.2 GA, 5.0.0 GA)
     */

    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.catalina.core.ApplicationContext", 
        new ServletContextImplClassBytecodeProcessor());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.catalina.loader.WebappLoader", 
        new WebappLoaderCBP());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.catalina.core.ApplicationContextFacade", 
        new ApplicationContextFacadeCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.catalina.servlets.DefaultServlet", 
        new DefaultServletCBP());
    
    /*
     * JASPER - Seeing changed classes in scriptlets
     *  
     * TOMCAT 6.x (Tested on 6.0.14)
     */
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.jasper.servlet.JasperLoader", 
        new JasperLoaderCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.jasper.compiler.JspRuntimeContext", 
        new JspRuntimeContextCBP());
    
    /*
     * JASPER - Removing JSPs
     *  
     * JETTY 6.1 (Tested on 6.1.12)
     * TOMCAT 5.5 (Tested on 5.5.27)
     */
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.jasper.compiler.Compiler", 
        new CompilerCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "org.apache.jasper.servlet.JspServlet", 
        new JspServletCBP());
    
    // WEBLOGIC 8.1, 9.2, 10.3 (Tested on 8.1.5, 9.2.2, 10.3)

    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.JspClassLoader", 
        new WeblogicJspClassLoaderCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.TagFileClassLoader", 
        new WeblogicTagFileClassLoaderCBP());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.internal.WebAppServletContext", 
        new WebAppServletContextCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.internal.ServletStubImpl", 
        new ServletStubImplCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.JspStub", 
        new JspStubCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.jsp.internal.CPL", 
        new WeblogicCPLCBP());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.TagFileHelper", 
        new TagFileHelperCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.JavelinxTagFileStaleChecker", 
        new JavelinxTagFileStaleCheckerCBP());
    
    // JSP updates in production mode
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.internal.WebAppConfigManager", 
        new WebAppConfigManagerCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "weblogic.servlet.jsp.JspConfig", 
        new JspConfigCBP());

    // OC4J 9.x, 10.x (Tested on 9.0.4, 10.1.2.0.2, 10.1.3.3.0, 10.1.3.4.0)

    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "oracle.jsp.runtimev2.JspClassLoader", 
        new OC4JJspClassLoaderCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "oracle.jsp.app.JspJavacCompiler", 
        new OC4JJspJavacCompilerCBP());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.evermind.server.http.HttpApplication", 
        new HttpApplicationCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.evermind.server.http.deployment.ServletMapping", 
        new ServletMappingCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.evermind.server.http.deployment.ServletDescriptor", 
        new ServletDescriptorCBP());
    
    // IBM WAS 6.1 (Tested on 6.1.0.0)

    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.ibm.ws.webcontainer.webapp.WebApp",
        new WebappCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.ibm.ws.jsp.webcontainerext.AbstractJSPExtensionServletWrapper",
        new AbstractJSPExtensionServletWrapperCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
//        cl,
        "com.ibm.ws.jsp.JspOptions",
        new JspOptionsCBP());
    
    // CAUCHO RESIN (Tested on 3.0.24)
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "com.caucho.server.webapp.Application",
        new ResinApplicationCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "com.caucho.jsp.Page",
        new PageCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "com.caucho.servlets.FileServlet",
        new DefaultServletCBP());
    
    // SAP NetWeaver 7.0
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//      cl,
      "com.sap.engine.services.servlets_jsp.server.runtime.context.ServletContextImpl",
      new SapServletContextImplCBP());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//      cl,
      "com.sap.engine.services.servlets_jsp.server.runtime.context.ApplicationContext",
      new SapApplicationContext());
    
    IntegrationFactory.getInstance().addIntegrationProcessor(
//      cl,        
        "com.sap.engine.services.servlets_jsp.server.servlet.DefaultServlet",
        new DefaultServletCBP());    
    
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
    return crs.getClassResource("javax.servlet.ServletContext") != null;
  }
 
  public String getId() {
    return "servlet_integration_plugin";
  }
  
  public String getName() {
    return null;
  }
  
  public String getDescription() {
    return null;
  }

  public String getAuthor() {
    return null;
  }

  public String getWebsite() {
    return null;
  }  
}
