package org.zeroturnaround.javarebel.integration.servlet.jasper;


import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.apache.jasper.compiler.JspRuntimeContext</code>
 * by integration JSP compiler class path with JavaRebel.
 * <p>
 * This enables to invoke new methods etc in JSP scriptlets
 * if <code>rebel.xml</code> is used for the application
 * and an external Java compiler is used for JSPs.
 * 
 * @author Rein Raudjärv
 * 
 * @see JasperLoaderCBP
 */
public class JspRuntimeContextCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");
    
    ctClass.getDeclaredMethod("initClassPath").instrument(new ExprEditor() {
      public void edit(MethodCall m) throws CannotCompileException {
        if (m.getClassName().equals("java.net.URLClassLoader")
            && m.getMethodName().equals("getURLs"))
          m.replace(
          		"{" +
          		"  URL[] result = $proceed($$);" +
          		
          		"  try {" +
          		"    Integration integration = IntegrationFactory.getInstance();" +
          		"    URL[] urls = integration.getRebelURLs(parentClassLoader, false);" +
          		"    if (urls != null) {" +
          		"      result = ResourceUtil.concat(urls, result);" +
          		"    }" +
          		"  } catch (Throwable t) {" +
          		"    LoggerFactory.getInstance().error(t);" +
          		"  }" +

          		"  $_ = result;" +
          		"}");
      }
    });
  }
}
