package org.zeroturnaround.javarebel.integration.servlet.oc4j;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>oracle.jsp.runtimev2.JspClassLoader</code>
 * to instrument JSP pages in OC¤J 9.x.
 * <p>
 * This enables to invoke new methods etc in JSP scriptlets.
 * 
 * @author Rein Raudjärv
 */
public class OC4JJspClassLoaderCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.util");
    
    if (isOC4J9(ctClass)) {
      processOC4J9(cp, cl, ctClass);
    }
    else if (hasClassMap(ctClass)) {
      /*
       * Do not create new JspClassLoader
       * but reinitialize JSP classes to update constant pools
       */
      ctClass.getDeclaredMethod("reinstantiate").setBody(
          "{" +
          "  ReinitClassReloadListener.addClasses(classMap.keySet());" +
          "  return this;" +
          "}");
    }
  }
    
  private boolean isOC4J9(CtClass ctClass) {
    try {
      ctClass.getDeclaredMethod("loadClass");
      return true;
    } catch (NotFoundException e) {
      return false;
    }
  }  
  
  private boolean hasClassMap(CtClass ctClass) {
    try {
      ctClass.getDeclaredField("classMap");
      return true;
    } catch (NotFoundException e) {
      return false;
    }
  }  
  
  private void processOC4J9(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.io");
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("java.util.zip");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.support");
    cp.importPackage("org.zeroturnaround.javarebel.integration.generic");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");
    
    ctClass.addInterface(cp.get("org.zeroturnaround.javarebel.ClassResourceSource"));
    
    CtConstructor[] constructors = ctClass.getConstructors();
    for (int i = 0; i < constructors.length; i++) {
      constructors[i].insertAfter(
      "IntegrationFactory.getInstance().registerClassLoader($0, (ClassResourceSource) $0);");
    }
    
    /*
     * By default JspClassLoader loads only JSP classes.
     * So the RebelLocator would not be loaded.
     * 
     * We make the exception and try to find the RebelLocator as well.
     */
    ctClass.getDeclaredMethod("loadClass").insertBefore(
        "{" +
        "  synchronized ($0) {" + 
        "    Class result =" + 
        "      findLoadedClass($1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "    result = " + 
        "      IntegrationFactory.getInstance().findReloadableClass($0, $1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "  }" +
    "}");
    
    ctClass.addMethod(CtNewMethod.make(
        "public Resource getLocalResource(String name) {" +
        "  Enumeration en = this.repository.elements();" +
        "  while (en.hasMoreElements()) {" +
        "    File rep = (File) en.nextElement();" +
        
        "    if (rep.isDirectory()) {" +
        "      File file = new File(rep, name);" +
        "      if (file.exists()) {" +
        "        try {" +
        "          URL url = makeURL(file);" +
        "          return new FileResource(url);" +
        "        } catch (MalformedURLException e) {" +
        "          LoggerFactory.getInstance().error(e);" +
        "          return null;" +
        "        }" +
        "      }" +
        "    }" +
        
        "    else if (rep.isFile()) {" +
        "      ZipFile zipFile = new ZipFile(rep);" +
        "      try {" +
        "        ZipEntry zipEntry = zipFile.getEntry(name);" +
        "        if (zipEntry != null) {" +
        "          try {" +
        "            URL url = ResourceUtil.makeURL(rep, name);" +
        "            return new URLResource(url);" +
        "          } catch (MalformedURLException e) {" +
        "            LoggerFactory.getInstance().error(e);" +
        "            return null;" +
        "          }" +
        "          " +
        "        }" +
        "      } finally {" +
        "        zipFile.close();" +
        "      }" +
        "    }" +
        
        "  }" +
        
        "  return null;" +
        "}", ctClass));
    
    ctClass.addMethod(CtNewMethod.make(
        "public Resource getClassResource(String className) {" +
        "  return ResourceUtil.getClassResource(this, className);" +
        "}", ctClass));
    
    ctClass.addMethod(CtNewMethod.make(
        "public Resource[] getLocalResources(String name) {" +
        "  return ResourceUtil.asArray(getLocalResource(name));" +
        "}", ctClass));
    
    // Do not create new JspClassLoader
    ctClass.getDeclaredMethod("reinstantiate").setBody("{ return this; }");    
  }
  
}
