package org.zeroturnaround.javarebel.integration.servlet.oc4j;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>oracle.jsp.app.JspJavacCompiler</code>.
 * 
 * @author Rein Raudjärv
 */
public class OC4JJspJavacCompilerCBP extends JavassistClassBytecodeProcessor {

  private static final String PREPEND_CLASSPATH_FIELD_NAME = "rebelClassPath";
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.io");
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");
    
    ctClass.addField(CtField.make("private java.lang.String " + PREPEND_CLASSPATH_FIELD_NAME + ";", ctClass));
    
    CtMethod m = ctClass.getDeclaredMethod("compile", cp.get(new String[] {
        "java.lang.String",
        "int",
        "java.lang.String",
        "oracle.jsp.provider.JspResourceProvider",
        "oracle.jsp.provider.JspResourceProvider",
        "oracle.jsp.parse.Jsp2JavaParms"
        }));
    
    // Get class path prepend value
    m.insertBefore(
            "{" +
            "  try {" +
            "    ClassLoader cl = $6.classLoader.getParent();" +
            "    URL[] urls = IntegrationFactory.getInstance().getRebelURLs(cl, false);" +
            "    if (urls != null) {" +
            "      " + PREPEND_CLASSPATH_FIELD_NAME + " = ResourceUtil.toClassPath(urls);" +
            "    }" +
            "  } catch (Throwable t) {" +
            "    LoggerFactory.getInstance().error(t);" +
            "  }" +
            "}");

    // Apply class path prepend value
    m.instrument(new ExprEditor() {
      public void edit(MethodCall m) throws CannotCompileException {
        if (m.getClassName().equals("com.evermind.compiler.LinkedCompiler") && m.getMethodName().equals("compile"))
          m.replace(
              "{" +
              "  try {" +
              "    String prepend = " + PREPEND_CLASSPATH_FIELD_NAME + ";" +
              "    if (prepend != null) {" +
              "      Map params = $4;" +
              "      String result = (String) params.get(\"classpath\");" +
              "      if (result == null) {" +
              "        result = prepend;" +
              "      } else {" +
              "        result = prepend + File.pathSeparator + result;" +
              "      }" +
              "      params.put(\"classpath\", result);" +
              "    }" +
              "  } catch (Throwable t) {" +
              "    LoggerFactory.getInstance().error(t);" +
              "  }" +

              "  $_ = $proceed($$);" +
              "}");
      }
    });
    
  }  
  
}
