package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>weblogic.servlet.jsp.JspStub<code>.
 * 
 * @author Rein Raudjärv
 */
public class JspStubCBP extends JavassistClassBytecodeProcessor {
  
  private static final String CLASS_LOADER_FIELD_NAME = "__rebel_cl";
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    if (hasGetNewClassLoader(ctClass)) {
      // WLS 9.x or newer
      cp.importPackage("org.zeroturnaround.javarebel");
      cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.util");
      
      /*
       * Do not create a new JspClassLoader if a JSP has been changed. 
       */
      ctClass.getDeclaredMethod("getNewClassLoader").setBody(
          "{" +
          "  ReinitClassReloadListener.addClass(className);" +
          "  return getClassLoader();" +
          "}");
    }
    else {
      // WLS 8.x (not tested with older)
      preserveClassLoader(ctClass);
    }
  }
  
  private boolean hasGetNewClassLoader(CtClass ctClass) {
    try {
      ctClass.getDeclaredMethod("getNewClassLoader");
      return true;
    } catch (NotFoundException e) {
      return false;
    }
  }
  
  private void preserveClassLoader(CtClass ctClass) throws Exception {
    ctClass.addField(CtField.make("private weblogic.servlet.jsp.JspClassLoader " + CLASS_LOADER_FIELD_NAME + ";", ctClass));
  
    CtMethod origMethod = ctClass.getDeclaredMethod("getClassLoader");
    CtMethod copyMethod = CtNewMethod.copy(origMethod, "_getClassLoader", ctClass, null);
    ctClass.addMethod(copyMethod);
    
    origMethod.setBody("{" +
          "  if (" + CLASS_LOADER_FIELD_NAME + " != null)" +
          "    return " + CLASS_LOADER_FIELD_NAME + ";" +

          "  weblogic.servlet.jsp.JspClassLoader result = _getClassLoader();" +
          "  " + CLASS_LOADER_FIELD_NAME + " = result;" +

          "  return result;" +
          "}");
  }
  
}
