package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Map;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>weblogic.servlet.jsp.TagFileHelper<code>.
 * This enables to reload tag files without recreating any class loaders.
 * 
 * @author Rein Raudjärv
 */
public class TagFileHelperCBP extends JavassistClassBytecodeProcessor {
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.util");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.util");
    cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.weblogic");
    
    /*
     * Do not recreate TagFileClassLoader but reinitialize classes (to update constants).
     */
    ctClass.getDeclaredMethod("reloadTagFileClassLoader").setBody(
        "{" +
        "  try {" +
        "    Collection classNames = TagFileHelperCBP.getClassNames(tagFileClassLoader);" +
        "    ReinitClassReloadListener.addClasses(classNames);" +
        "  } catch (Throwable t) {" +
        "    LoggerFactory.getInstance().error(t);" +        
        "  }" +
        "}");
  }
  
  /**
   * Returns the names of classes loaded by the given <code>ChangeAwareClassLoader</code>.
   * <p>
   * A private field <code>modTimes</code> contains a <code>Map</code>.
   * The keys of this map are returned. 
   */
  public static Collection getClassNames(ClassLoader changeAwareClassLoader) throws Exception {
    Field field = Helper.FIELD_MOD_TIMES;
    if (field == null)
      return null; // Could not get the field
    
    Map map = (Map) field.get(changeAwareClassLoader);
    return map.keySet();
  }
  
  private static class Helper {
    
    private static final Field FIELD_MOD_TIMES = getModTimes();

    private static Field getModTimes() {
      try {
        Class changeAwareClassLoader = Class.forName("weblogic.utils.classloaders.ChangeAwareClassLoader");
        Field field = changeAwareClassLoader.getDeclaredField("modTimes");
        field.setAccessible(true);
        return field;
      }
      catch (Throwable t) {
        LoggerFactory.getInstance().error(t);
      }
      return null;
    }
    
  }
  
}
