package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>weblogic.jsp.internal.CPL<code>.
 * <p>
 * This enables to call new methods from scriptlets
 * when using <code>rebel.xml</code> in WLS 10.x or newer.
 * 
 * @author Rein Raudjärv
 */
public class WeblogicCPLCBP extends JavassistClassBytecodeProcessor {

  private static final String METHOD_INIT_CLASS_PATH = "_initClassPath";
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");  
    
    /*
     * Prepend rebel URLs to the class path.
     */
    ctClass.addMethod(CtNewMethod.make("private String " + METHOD_INIT_CLASS_PATH + "(weblogic.utils.classloaders.GenericClassLoader gcl) {" +
        "  String result = gcl.getClassPath();" +

        "  try {" +
        "    ClassLoader cl = gcl.getParent();" +

        "    Integration integration = IntegrationFactory.getInstance();" +
        "    java.net.URL[] urls = integration.getRebelURLs(cl, false);" +
        "    if (urls != null) {" +
        "      result = ResourceUtil.concatClassPath(ResourceUtil.toClassPath(urls), result);" +
        "      result = weblogic.utils.io.FilenameEncoder.cleanClasspath(result);" +
        "    }" +
        "  } catch (Throwable t) {" +
        "    LoggerFactory.getInstance().error(t);" +
        "  }" +

        "  return result;" +
        "}", ctClass));
    
    ctClass.getDeclaredMethod("setClassLoader").instrument(new ExprEditor() {
      public void edit(MethodCall m) throws CannotCompileException {
        if (m.getClassName().equals("weblogic.utils.classloaders.GenericClassLoader")
            && m.getMethodName().equals("getClassPath"))
          m.replace("{ $_ = " + METHOD_INIT_CLASS_PATH + "($0); }");
      }
    });
  }

}
