package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>weblogic.servlet.jsp.TagFileClassLoader</code>.
 * 
 * @author Rein Raudjärv
 */
public class WeblogicTagFileClassLoaderCBP extends JavassistClassBytecodeProcessor {

  private static final String REBEL_LOCATER_PREFIX = "com.zeroturnaround.javarebel.RebelLocator$$";
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    /*
     * By default TagFileClassLoader loads only Tag file classes.
     * So the RebelLocator would not be loaded.
     * 
     * We make the exception and try to find the RebelLocator as well.
     */
    ctClass.getDeclaredMethod("findClass").insertBefore(
      "{" +
      "  if ($1.startsWith(\"" + REBEL_LOCATER_PREFIX + "\")) {" +
      "    Class clazz = findLoadedClass($1);" +
      "    if (clazz != null) {" +
      "      return clazz;" +
      "    }" +
      
      "    return super.findClass($1);" +
      "  }" +
      "}");

    /*
     * Overrides the method added by JavaRebel
     * This re-enables ChangeAwareClassLoader.upToDate() method.
     */
    ctClass.addMethod(CtNewMethod.make(
        "protected boolean __isAlwaysUpToDate() { return false; }", ctClass));
  }
}
